/***************************************************************************
 *   Copyright (C) 2012-2013 by Eike Hein <hein@kde.org>                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

function horizontalMargins() {
    return taskFrame.margins.left + taskFrame.margins.right;
}

function verticalMargins() {
    return taskFrame.margins.top + taskFrame.margins.bottom;
}

function adjustMargin(height, margin) {
    var available = height - verticalMargins();

    if (available < units.iconSizes.small) {
        return Math.floor((margin * (units.iconSizes.small / available)) / 3);
     }

    return margin;
}

function launcherLayoutTasks() {
    return Math.round(backend.tasksModel.launcherCount / Math.floor(preferredMinWidth() / launcherWidth()));
}

function launcherLayoutWidthDiff() {
    return (launcherLayoutTasks() * taskWidth()) - (backend.tasksModel.launcherCount * launcherWidth());
}

function logicalTaskCount() {
    var count = tasks.vertical ? backend.tasksModel.count : backend.tasksModel.count - backend.tasksModel.launcherCount;

    return Math.max(backend.tasksModel.count ? 1 : 0, count);
}

function maxStripes() {
    var length = tasks.vertical ? taskList.width : taskList.height;
    var minimum = tasks.vertical ? preferredMinWidth() : preferredMinHeight();

    return Math.min(plasmoid.configuration.maxStripes, Math.max(1, Math.floor(length / minimum)));
}

function tasksPerStripe() {
    if (plasmoid.configuration.forceStripes) {
        return Math.ceil(logicalTaskCount() / maxStripes());
    } else {
        var length = tasks.vertical ? taskList.height : taskList.width;
        var minimum = tasks.vertical ? preferredMinHeight() : preferredMinWidth();

        return Math.floor(length / minimum);
    }
}

function calculateStripes() {
    var stripes = plasmoid.configuration.forceStripes ? plasmoid.configuration.maxStripes : Math.min(plasmoid.configuration.maxStripes, Math.ceil(logicalTaskCount() / tasksPerStripe()));

    return Math.min(stripes, maxStripes());
}

function full() {
    return (maxStripes() == calculateStripes());
}

function optimumCapacity(width, height) {
    var length = tasks.vertical ? height : width - (backend.tasksModel.launcherCount * launcherWidth());
    var maximum = tasks.vertical ? preferredMaxHeight() : preferredMaxWidth();

    return (Math.ceil(length / maximum) * maxStripes()) + backend.tasksModel.launcherCount;
}

function layoutWidth() {
    if (plasmoid.configuration.forceStripes && !tasks.vertical) {
        return Math.min(tasks.width, Math.max(preferredMaxWidth(), tasksPerStripe() * preferredMaxWidth() + Math.ceil(tasksModel.launcherCount / maxStripes()) * launcherWidth()));
    } else {
        return tasks.width;
    }
}

function layoutHeight() {
    if (plasmoid.configuration.forceStripes && tasks.vertical) {
        return Math.min(tasks.height, Math.max(preferredMaxHeight(), tasksPerStripe() * preferredMaxHeight()));
    } else {
        return tasks.height;
    }
}

function preferredMinWidth() {
    var width = launcherWidth();

    if (!tasks.vertical && !tasks.iconsOnly) {
        width += (units.smallSpacing * 2) + (theme.mSize(theme.defaultFont).width * 12);
    }

    return width;
}

function preferredMaxWidth() {
    return preferredMinWidth();
}

function preferredMinHeight() {
    // TODO FIXME UPSTREAM: Port to proper font metrics for descenders once we have access to them.
    return theme.mSize(theme.defaultFont).height + 4;
}

function preferredMaxHeight() {
    return verticalMargins() + Math.min(units.iconSizes.small * 3, theme.mSize(theme.defaultFont).height * 3);
}

function taskWidth() {
    if (tasks.vertical) {
        return Math.floor(taskList.width / calculateStripes());
    } else {
        if (full() && Math.max(1, logicalTaskCount()) > tasksPerStripe()) {
            return Math.floor(taskList.width / Math.ceil(logicalTaskCount() / maxStripes()));
        } else {
            return Math.min(preferredMaxWidth(), Math.floor(taskList.width / Math.min(logicalTaskCount(), tasksPerStripe())));
        }
    }
}

function taskHeight() {
    if (tasks.vertical) {
        if (full() && Math.max(1, logicalTaskCount()) > tasksPerStripe()) {
            return Math.floor(taskList.height / Math.ceil(logicalTaskCount() / maxStripes()));
        } else {
            return Math.min(preferredMaxHeight(), Math.floor(taskList.height / Math.min(logicalTaskCount(), tasksPerStripe())));
        }
    } else {
        return Math.floor(taskList.height / calculateStripes());
    }
}

function launcherWidth() {
    return (tasks.height + horizontalMargins())
        - (adjustMargin(tasks.height, taskFrame.margins.top) + adjustMargin(tasks.height, taskFrame.margins.bottom));
}

function layout(container) {
    if (backend.tasksModel.count == 0) {
        return;
    }

    var width = taskWidth();
    var height = taskHeight();
    var lWidth = launcherWidth();
    var pMaxWidth = preferredMaxWidth();
    var perStripe = Math.floor(taskList.width / width);

    var item;
    var task = 0;
    var launcherCountForStripe = new Array();
    var taskCountForStripe = new Array();
    var launcherCount = 0;
    var taskCount = 0;

    for (var i = 0; i < container.count; ++i) {
        item = container.itemAt(i);

        if (!item) {
            return;
        }

        if (item.isLauncher) {
            ++launcherCount;
        } else {
            ++task;
            ++taskCount;
        }

        if (task == perStripe) {
            task = 0;
            launcherCountForStripe.push(launcherCount);
            taskCountForStripe.push(taskCount);
            launcherCount = 0;
            taskCount = 0;
        }
    }

    launcherCountForStripe.push(launcherCount);
    taskCountForStripe.push(taskCount);

    var stripe = 0;
    task = 0;
    launcherCount = (calculateStripes() > 1) ? launcherCountForStripe[stripe] : backend.tasksModel.launcherCount;
    taskCount = taskCountForStripe[stripe];

    var diff = Math.ceil((launcherCount * lWidth) / taskCount);
    var volume = (taskCount * (width - diff)) + (launcherCount * lWidth);
    diff = diff - Math.max(0, Math.floor((taskList.width - volume) / taskCount));

    for (var i = 0; i < container.count; ++i) {
        item = container.itemAt(i);
        item.visible = true;

        if (!tasks.vertical) {
            if (item.isLauncher) {
                item.width = lWidth;
            } else {
                ++task;
                item.width = Math.min(pMaxWidth, width - diff);
            }

            if (task == perStripe) {
                ++stripe;
                task = 0;

                if (launcherCountForStripe[stripe] != launcherCount) {
                    launcherCount = launcherCountForStripe[stripe];
                    taskCount = taskCountForStripe[stripe];
                    diff = Math.ceil((launcherCount * lWidth) / taskCount);
                    volume = (taskCount * (width - diff)) + (launcherCount * lWidth);
                    diff = diff - Math.max(0, Math.floor((taskList.width - volume) / taskCount));
                }
            }
        } else {
            item.width = width;
        }

        item.height = height;
    }
}
